#
# Product Manager
#
# (C) Copyright 2000,2001 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

from Globals import DTMLFile
from OFS.Folder import Folder
from Acquisition import Implicit
import urllib
import magic
import os
import string

#
# Hacked up or otherwise modified archive handlers
#
from Products.ProductManager.Archivers.Tar import TarFile, TarGzFile
from Products.ProductManager.Archivers.local_zipfile import ZipFile, ZipInfo

class ProductInstaller(Implicit):

	def __init__(self, category, version, name):
		self.category=category
		self.version=version
		self.name=name

	def install(self):
		print self.name, self.version, self.category
		p = self.getPackage(self.name , self.version, self.category)[0]
		print p.download_sites
		
		for i in p.download_sites:
			try:
				fname, headers = urllib.urlretrieve(i)
			except:
				continue
			# zope.org lies about the type of the file..
			# more likely it simply doesn't know, things seem to be
			# application/octet-stream, not very useful.

			# We need to extend magic to know about zexp files if possible
			fileType= magic.file(fname)
			
			if fileType == 'application/zip':
				x = ZipFile(fname, 'r')
				self.dealWithZip(x)
			elif fileType == 'application/x-gzip':
				x = TarGzFile(fname)
				self.dealWithTar(x)
			elif fileType == 'application/x-gtar':
				x = TarFile(fname)
				self.dealWithTar(x)
			elif fileType == 'application/x-tar':
				x = TarFile(fname)
				self.dealWithTar(x)
			else:
				os.unlink(fname)
				raise 'Unknown File Type', fileType

			os.unlink(fname)
			return 1
		return 0

			
	def dealWithTar(self, archive):
		n = archive.getNumEntries()
		print "Archive consists of %d entries."%(n)
		e = archive.getEntry(0)

		print "Root entry is %s"%(e.name)
		if e.name[:20] == 'lib/python/Products/':
			PRODUCT_PATH=INSTANCE_HOME
		else:
			PRODUCT_PATH=os.path.join(SOFTWARE_HOME, 'Products')
		print "PRODUCTS_HOME is %s"%(PRODUCT_PATH)
		
		if e.isFile():
			# Someone did tar cvf /blah/foo/* 
			print "Root entry is a file?"
			# *sigh* we have to manually make the root directory probably...
			# And since the tar archive doesn't have a friggen entry we have
			# no mode information for it...
			dirToMake=string.join(string.split(e.name, '/')[:-1], '/')
			dirToMake=os.path.join(PRODUCT_PATH, dirToMake)
			print "Creating %s"%(dirToMake)
			try:
				os.makedirs(dirToMake, 0755)
			except:
				pass

		for i in range(n):
			e = archive.getEntry(i)
			e.extractToFileSystem(PRODUCT_PATH)
		return 1
		
	def dealWithZip(self, archive):
		ie = archive.infolist()		
		n = len(ie)
		print "Archive consists of %d entries."%(n)
		ie = archive.infolist()
		e = ie[n-1]
		print "Root entry is %s"%(e.filename)

		if e.filename[:20] == 'lib/python/Products':
			PRODUCT_PATH=INSTANCE_HOME
		else:
			PRODUCT_PATH=os.path.join(SOFTWARE_HOME, 'Products')

		print "PRODUCTS_HOME is %s"%(PRODUCT_PATH)

		if e.isFile():
			print "Root entry is a file?"
			# *sigh* we have to manually make the root directory probably...
			dirToMake=string.join(string.split(e.filename, '/')[:-1], '/')
			dirToMake=os.path.join(PRODUCT_PATH, dirToMake)
			print "Creating %s"%(dirToMake)
			try:
				os.makedirs(dirToMake, 0755)
			except:
				pass
		
		for i in range(n):
			e = ie.pop()
			e.extractToFileSystem(archive, PRODUCT_PATH)
		
